@@ -7,9 +7,10 @@ import (
 
 // statusCmd represents the status command
 var statusCmd = &cobra.Command{
-	Use:   "status",
+	Use:   "status [profile]",
 	Short: "show the status of Colima",
 	Long:  `Show the status of Colima`,
+	Args:  cobra.MaximumNArgs(1),
 	RunE: func(cmd *cobra.Command, args []string) error {
 		return newApp().Status()
 	},
