@@ -1,6 +1,9 @@
 package kubernetes
 
 import (
+	"context"
+	"encoding/json"
+	"fmt"
 	"strings"
 	"time"
 
@@ -12,7 +15,13 @@ import (
 )
 
 // Name is container runtime name
-const Name = "kubernetes"
+
+const (
+	Name           = "kubernetes"
+	DefaultVersion = "v1.23.6+k3s1"
+
+	configKey = "kubernetes_config"
+)
 
 func newRuntime(host environment.HostActions, guest environment.GuestActions) environment.Container {
 	return &kubernetesRuntime{
@@ -23,7 +32,7 @@ func newRuntime(host environment.HostActions, guest environment.GuestActions) en
 }
 
 func init() {
-	environment.RegisterContainer(Name, newRuntime)
+	environment.RegisterContainer(Name, newRuntime, true)
 }
 
 var _ environment.Container = (*kubernetesRuntime)(nil)
@@ -42,61 +51,119 @@ func (c kubernetesRuntime) isInstalled() bool {
 	// it is installed if uninstall script is present.
 	return c.guest.RunQuiet("command", "-v", "k3s-uninstall.sh") == nil
 }
+func (c kubernetesRuntime) isVersionInstalled(version string) bool {
+	// validate version change via cli flag/config.
+	out, err := c.guest.RunOutput("k3s", "--version")
+	if err != nil {
+		return false
+	}
+	return strings.Contains(out, version)
+}
 
-func (c kubernetesRuntime) Running() bool {
+func (c kubernetesRuntime) Running(ctx context.Context) bool {
 	return c.guest.RunQuiet("sudo", "service", "k3s", "status") == nil
 }
 
 func (c kubernetesRuntime) runtime() string {
 	return c.guest.Get(environment.ContainerRuntimeKey)
 }
-func (c kubernetesRuntime) kubernetesVersion() string { //nolint:unused
-	return c.guest.Get(environment.KubernetesVersionKey)
+
+func (c kubernetesRuntime) config() config.Kubernetes {
+	conf := config.Kubernetes{Version: DefaultVersion}
+	if b := c.guest.Get(configKey); b != "" {
+		_ = json.Unmarshal([]byte(b), &conf)
+	}
+	return conf
 }
 
-func (c *kubernetesRuntime) Provision() error {
-	log := c.Logger()
-	a := c.Init()
+func (c kubernetesRuntime) setConfig(conf config.Kubernetes) error {
+	b, err := json.Marshal(conf)
+	if err != nil {
+		return fmt.Errorf("error encoding kubernetes config to json: %w", err)
+	}
 
-	if !c.isInstalled() {
-		// k3s
-		a.Stage("downloading and installing")
-		installK3s(c.host, c.guest, a, log, c.runtime())
+	return c.guest.Set(configKey, string(b))
+}
+
+func (c *kubernetesRuntime) Provision(ctx context.Context) error {
+	log := c.Logger(ctx)
+	a := c.Init(ctx)
+	if c.Running(ctx) {
+		return nil
+	}
+
+	appConf, ok := ctx.Value(config.CtxKey()).(config.Config)
+	runtime := appConf.Runtime
+	conf := appConf.Kubernetes
+
+	if !ok {
+		// this should be a restart/start while vm is active
+		// retrieve value in the vm
+		runtime = c.runtime()
+		conf = c.config()
+	}
+
+	if c.isVersionInstalled(conf.Version) {
+		// runtime has changed, ensure the required images are in the registry
+		if currentRuntime := c.runtime(); currentRuntime != "" && currentRuntime != runtime {
+			a.Stagef("changing runtime to %s", runtime)
+			installK3sCache(c.host, c.guest, a, log, runtime, conf.Version)
+		}
+		// other settings may have changed e.g. ingress
+		installK3sCluster(c.host, c.guest, a, runtime, conf.Version, conf.Ingress)
+	} else {
+		if c.isInstalled() {
+			a.Stagef("version changed to %s, downloading and installing", conf.Version)
+		} else {
+			if ok {
+				a.Stage("downloading and installing")
+			} else {
+				a.Stage("installing")
+			}
+		}
+		installK3s(c.host, c.guest, a, log, runtime, conf.Version, conf.Ingress)
 	}
 
 	// this needs to happen on each startup
-	if c.runtime() == containerd.Name {
+	switch runtime {
+	case containerd.Name:
 		installContainerdDeps(c.guest, a)
+	case docker.Name:
+		a.Retry("waiting for docker cri", time.Second*2, 5, func(int) error {
+			return c.guest.Run("sudo", "service", "cri-dockerd", "start")
+		})
 	}
 
+	// provision successful, now we can persist the version
+	a.Add(func() error { return c.setConfig(conf) })
+
 	return a.Exec()
 }
 
-func (c kubernetesRuntime) Start() error {
-	log := c.Logger()
-	a := c.Init()
-	if c.Running() {
+func (c kubernetesRuntime) Start(ctx context.Context) error {
+	log := c.Logger(ctx)
+	a := c.Init(ctx)
+	if c.Running(ctx) {
 		log.Println("already running")
 		return nil
 	}
 
-	a.Stage("starting")
-
 	a.Add(func() error {
-		defer time.Sleep(time.Second * 5)
 		return c.guest.Run("sudo", "service", "k3s", "start")
 	})
+	a.Retry("", time.Second*2, 10, func(int) error {
+		return c.guest.RunQuiet("kubectl", "cluster-info")
+	})
 
 	if err := a.Exec(); err != nil {
 		return err
 	}
 
-	return c.provisionKubeconfig()
+	return c.provisionKubeconfig(ctx)
 }
 
-func (c kubernetesRuntime) Stop() error {
-	a := c.Init()
-	a.Stage("stopping")
+func (c kubernetesRuntime) Stop(ctx context.Context) error {
+	a := c.Init(ctx)
 	a.Add(func() error {
 		return c.guest.Run("k3s-killall.sh")
 	})
@@ -172,9 +239,8 @@ func (c kubernetesRuntime) runningContainerIDs() string {
 	return strings.ReplaceAll(ids, "\n", " ")
 }
 
-func (c kubernetesRuntime) Teardown() error {
-	a := c.Init()
-	a.Stage("deleting")
+func (c kubernetesRuntime) Teardown(ctx context.Context) error {
+	a := c.Init(ctx)
 
 	if c.isInstalled() {
 		a.Add(func() error {
@@ -189,9 +255,6 @@ func (c kubernetesRuntime) Teardown() error {
 	})
 
 	c.teardownKubeconfig(a)
-	a.Add(func() error {
-		return c.guest.Set(kubeconfigKey, "")
-	})
 
 	return a.Exec()
 }
@@ -200,7 +263,7 @@ func (c kubernetesRuntime) Dependencies() []string {
 	return []string{"kubectl"}
 }
 
-func (c kubernetesRuntime) Version() string {
-	version, _ := c.host.RunOutput("kubectl", "--context", config.Profile().ID, "version", "--short")
+func (c kubernetesRuntime) Version(ctx context.Context) string {
+	version, _ := c.host.RunOutput("kubectl", "--context", config.CurrentProfile().ID, "version", "--short")
 	return version
 }
